# Copyright (C) 2011 Diego Pardilla Mata
#
#    This file is part of SpiderBOY.
#
#     SpiderBOY is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import itertools
import sys
import time

def spin():
    parts = ['|', '/', '-', '\\']

    cnt = 1
    for part in itertools.cycle(parts):
        #if cnt >= max:
        #   break
        sys.stdout.write(part)
        sys.stdout.flush()
        time.sleep(.1)
        sys.stdout.write('\b')
        cnt += 1


# Copyright (C) 2011 Diego Pardilla Mata
#
#    This file is part of SpiderBOY.
#
#     SpiderBOY is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pyPdf import PdfFileReader as Reader

def getPDFContent(path):
    content = ""
    # Load PDF into pyPDF
    pdf = Reader(file(path, "rb"))
    # Iterate pages
    for i in range(0, pdf.getNumPages()):
        # Extract text from page and add to content
        content += pdf.getPage(i).extractText() + "\n"
        # Collapse whitespace

def getPDFlinks (path,proto_filter):
    links = []
    pdf = Reader(file(path, "rb"))
    pages = pdf.getNumPages()
    key = '/Annots'
    uri = '/URI'
    anchor = '/A'
    for page in range(pages):
        pag = pdf.getPage(page)
        obj = pag.getObject()
        if obj.has_key(key):
            list_annots = obj[key]
            #print list_annots
            links = links + [ link.getObject()[anchor][uri] for link in list_annots
                     if (link.getObject().has_key(anchor) and link.getObject()[anchor].has_key(uri) and link.getObject()[anchor][uri].find(proto_filter)==0)]
    return links


#!/usr/bin/python

# Copyright (C) 2011 Diego Pardilla Mata
#
#	This file is part of SpiderBOY.
#
# 	SpiderBOY is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

import urllib2
import argparse
import tempfile
from BeautifulSoup import BeautifulSoup as Soup
import os
import signal
from color import colorstr
from pdf import getPDFlinks
from spinslash import spin



parser = argparse.ArgumentParser (description="Let's spiderBOY the Internet")
parser.add_argument ( 'url' , nargs=1 , help = 'target URL')
parser.add_argument ( '-n' , '--number-of-levels' , type=int , default=1 , help = 'how deep the spiderBOY program must track')
parser.add_argument ( '-p' , '--type-of-protocol' , type=str , default='http' , help = 'what type of protocol to use, only one (e.g. http or https or ftp)')
args = parser.parse_args()

# Input parameters
target_url = args.url.pop()
deep = args.number_of_levels
type_proto = args.type_of_protocol

# Function that get links
def get_links(target):
	user_agent = 'Mozilla/5.0 (X11;U;Linux x86_64;en-US) AppleWebKit/534.7 ( KHTML , like Gecko ) Chrome/7.0.517.41 Safari/534.7'
	opener = urllib2.build_opener()
	opener.addheaders = [('User-agent', user_agent)]
	# Get protocol
	if type_proto != 'file':
		proto_filter = type_proto + '://'
	else:
		proto_filter = 	type_proto + ':///'
	# Let's see if it is a pdf file
	if '.pdf' in target:
		raw_code = urllib2.urlopen(target.encode("utf-8")).read()
		opener.close()
		temp = tempfile.NamedTemporaryFile(prefix='spiderBOY_', suffix='.tmp', dir='/tmp', delete=False)
		temp.write(raw_code)
		temp.close()
		links = getPDFlinks(temp.name,proto_filter)
		
	else:	
		# Catch an URL exception	
		try:
			raw_code = opener.open(target.encode("utf-8")).read()
			opener.close()
		except urllib2.URLError, e:
			if hasattr(e, 'reason'):
				print colorstr('\nWe failed to reach a server.', 'RED')
				print colorstr('This url: ' + target + ' has a problem and the reason is: ' + repr(e.reason), 'RED')
			elif hasattr(e, 'code'):
				print colorstr('\nThe server couldn\'t fulfill the request.', 'RED')
				print colorstr('This url: ' + target + ' has a problem and the error code is: ' + repr(e.code), 'RED')
			links = []
		else:
			soup_code = Soup (raw_code)
			links = [ link ['href'] for link
					in soup_code.findAll ('a')
					if (link.has_key ('href') and (link['href'].find(proto_filter)==0))]
	return links

# Function that input is a list and an output is a dictionary with the level of proof
def read_list (elements_list,level,dict_out,proof):
	level = level + 1
	proof = proof - 1
	# Cover the list of links
	for node in elements_list:
		aux_list = get_links(node)
		# trace program
		#print 'node:' + node
		#print 'list:' + str(aux_list)
		#print 'deep:' + str(proof)
		if proof > 0:
			dict_out[node] = level 
			read_list(aux_list,level,dict_out,proof)
		else:
			dict_out[node] = level	
	return dict_out

# Function that paint the dictionary with {url, proof}
def paint_list (dict2print):
	element = '\n'
	aux_list = dict2print.keys()

	for key in aux_list:
		level = dict2print[key]
		for i in range(level): 
			element = element + '*'
		if '\n' in key:
			element = element + ' ' + str(key.encode("utf-8"));
		else:
			element = element + ' ' + str(key.encode("utf-8")) + '\n';
	print element
	# trace print dictionary
	#print 'Dictionary (url, level of proof):', dict2print
	return 0	

# Spinning slash for wait process
child_pid = os.fork()
if child_pid == 0:
	print ("Please wait "),
	spin()
else:
	# Initialize the list of links
	list_links = get_links(target_url)	

# Read the list of links
level = 0
dict_out = {}
out_dict = read_list (list_links,level,dict_out,deep)
# Paint the dictionary
paint_list(out_dict)
os.kill(child_pid, signal.SIGTERM)
os.waitpid(child_pid, 0) # need



# Copyright (C) 2011 Diego Pardilla Mata
#
#    This file is part of SpiderBOY.
#
#     SpiderBOY is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

CODE={
    'ENDC':0,  # RESET COLOR
    'BOLD':1,
    'UNDERLINE':4,
    'BLINK':5,
    'INVERT':7,
    'CONCEALD':8,
    'STRIKE':9,
    'GREY30':90,
    'GREY40':2,
    'GREY65':37,
    'GREY70':97,
    'GREY20_BG':40,
    'GREY33_BG':100,
    'GREY80_BG':47,
    'GREY93_BG':107,
    'DARK_RED':31,
    'RED':91,
    'RED_BG':41,
    'LIGHT_RED_BG':101,
    'DARK_YELLOW':33,
    'YELLOW':93,
    'YELLOW_BG':43,
    'LIGHT_YELLOW_BG':103,
    'DARK_BLUE':34,
    'BLUE':94,
    'BLUE_BG':44,
    'LIGHT_BLUE_BG':104,
    'DARK_MAGENTA':35,
    'PURPLE':95,
    'MAGENTA_BG':45,
    'LIGHT_PURPLE_BG':105,
    'DARK_CYAN':36,
    'AUQA':96,
    'CYAN_BG':46,
    'LIGHT_AUQA_BG':106,
    'DARK_GREEN':32,
    'GREEN':92,
    'GREEN_BG':42,
    'LIGHT_GREEN_BG':102,
    'BLACK':30,
}

def termcode(num):
    return '\033[%sm'%num

def colorstr(astr,color):
    return termcode(CODE[color])+astr+termcode(CODE['ENDC'])


# Copyright (C) 2011 Diego Pardilla Mata
#
#	This file is part of SpiderBOY.
#
# 	SpiderBOY is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from setuptools import setup, find_packages

def read(fname):
        return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup (name = "SpiderBOY",
	version = "1.0.0",
	packages = find_packages(),
	scripts = ['spiderboy.py'],
        py_modules = ['color', 'pdf', 'spinslash'],
	install_requires = ['BeautifulSoup', 'pyPdf'],
	package_data = {},
	author="Diego Pardilla",
	author_email="diegopardilla@gmail.com",
	description="A tipical crawler, returns a list of urls",
	license="GPL v3",
	keywords="crawler, spider",
        url="http://code.sidelab.es/projects/diegopardilla/wiki",
        long_description=read('README'),
	#long_description="This application returns a list of urls from a url and its level of depth.",
	download_url="git://gitorious.org/practicas_mswl/practicas_mswl_dt.git")
